//////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//
// PROGRAMA PROBAS MEDIDOR AMBIENTAL
// ---------------------------------------------------------------------------------------------------------------
// Este programa envía datos de sensores a un servidor remoto en forma de chamada http a un script php
//
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//
// bibliotecas de funcións usadas no programa

#include <SPI.h> // para conexión co módulo ethernet 
#include <Ethernet.h> // para conexión co módulo ethernet
#include <SoftwareSerial.h> // para crear un porto serie virtual
#include <EEPROM.h>
#include "html_funcions.h"

//////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//
// definicións de constantes do programa
//
// parámetros de comunicación serie
// porto serie hw:
#define SERIE_VELOCIDADE 9600
#define SERIE_FORMATO SERIAL_8N1

// ACTUALIZAR MAC
// se a constante seguinte está definida a lectura da eeprom forza unha escritura na que se cambia o byte6 da mac
// isto permite actualizar un valor anterior, porque non se pode facer a través do interfaz web
//#define local_mac5 0x73
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//
// variables para conexión ip
//
// dirección MAC do módulo wiznet
// macs de módulos wiznet wiz110sr: 00:08:DC:13:AE:5D, 00:08:DC:13:AE:5F
// 3 bytes iniciais da mac son do fabricante, 00:08:DC = Wiznet
// usar 00:08:DC:13:AE:xx (xx=0x5d e sucesivas para cada medidor)
// dec2021: para o byte 6 usar valores 0x61-0x78 para medidores 1-24 
byte local_mac[] = { 0x00, 0x08, 0xdc, 0x13, 0xae, 0x5d };

// configuración inicial local
/*
/*/
IPAddress local_ip  ( 192, 168,   2,  2 );
IPAddress local_mask( 255, 255, 255,  0 );
IPAddress dns_ip    ( 192, 168,   2,  1 );
IPAddress gw_ip     ( 192, 168,   2,  1 );

#define MEDIDOR_NUMERO  0       //número de medidor inválido por defecto
#define MEDIDOR_HOST "192.168.2.1" //servidor local (easyphp)
#define MEDIDOR_RUTA "datos.php" //ruta de datos do medidor (con script)
#define MEDIDOR_PORT 80 //usa o porto por defecto para o servidor interno e o remoto
//*/

// configuración inicial remota
/*/
IPAddress local_ip  (  10,  66, 216,165 );
IPAddress local_mask( 255, 255, 252,  0 );
IPAddress dns_ip    (  10, 180, 188, 75 );
IPAddress gw_ip     (  10,  66, 216,  1 );

#define MEDIDOR_NUMERO  2       //número de medidor inválido por defecto
#define MEDIDOR_HOST "codos.meteoproval.es" //servidor local (easyphp)
#define MEDIDOR_RUTA "datos/medidor2/datos.php" //ruta de datos do medidor (con script)
#define MEDIDOR_PORT 80 //usa o porto por defecto para o servidor interno e o remoto
//*/

// definición de nomes abreviados para datos de sensores
#define TEMP  datos[0]
#define HUM   datos[1]
#define CO2   datos[2]
#define LUM   datos[3]
#define PRES  datos[4]
#define VAR6  datos[5]
#define VAR7  datos[6]
#define VAR8  datos[7]
#define VAR9  datos[8]
#define VAR10 datos[9]

//////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Variables de xestión de comunicacións tcpip
EthernetServer server(MEDIDOR_PORT);
EthernetClient cliente; //usa unha única variable cliente para conexións entrantes ou saintes (non coinciden no tempo)

byte medidor_numero = MEDIDOR_NUMERO; //número do medidor
//variable datos para lectura de sensores
// variable 0 = temperatura ºC
// variable 1 = humidade relativa %
// variable 2 = nivel co2 ppm
// variable 3 = intensidade luminosa W/m2
// variable 4 = presión atmosférica mbar
// variables 5-9 = sen uso
unsigned int datos[10];

#define MAXBUF 250 //máximo número de caracteres procesados nas consultas ao microservidor
//(ten que ser inferior a 255 porque se manexa cun índice byte)
char linea[MAXBUF + 1]; //buffer de texto (último carácter 0 = fin de cadena)
char cad1[20], cad2[20]; //cadenas auxiliares para liñas html (números ou nomes curtos)
char medidor_host[41] = MEDIDOR_HOST; //lonxitude máxima 40 caracteres
char medidor_ruta[41] = MEDIDOR_RUTA; //inclúe nome de script ó final

//variable que garda o tempo da última conexión válida
unsigned long ultima_conexion = 0;
// máximo intervalo entre conexións en ms, cando se supera manda unha trama nula
// para controlar o funcionamento do enlace
const unsigned long intervalo_conexion = 60000L; //60 s = 1 minuto

//////////////////////////////////////////////////////////////////////////////////////////////////////////////////

void setup()
{

  // Open serial communications and wait for port to open:
  Serial.begin(9600);
  while (!Serial) {
    ; // wait for serial port to connect. Needed for native USB port only
  }
  Serial.println(F("Medidor Ambiental"));

  //  eeprom_escribe_configuracion();

  if (eeprom_lee_configuracion())
  { Serial.println(F("Eeprom OK"));
#ifdef local_mac5 //PARA MODIFICAR MAC ANTERIOR
    local_mac[5]=local_mac5; // cambiar só último byte
    eeprom_escribe_configuracion();
#endif
  }
  else
    Serial.println(F("Eeprom ERRO"));

  inicio_sensor_luz();
  inicio_co2();
  http_inicio();
  http_consulta(0); //manda trama nula inicial para control

}

//////////////////////////////////////////////////////////////////////////////////////////////////////////////////

void loop()
{
  // ENVÍO DATOS A SERVIDOR REMOTO
  //comproba tempo desde última conexión e se excede o valor prefixado
  //fai a lectura de sensores e envía datos ó servidor remoto
  //NOTA: camdo se activa a biblioteca SoftwareSerial cambia o temporizador (T0?) e
  //a función millis() vai máis lenta
  if ( (millis() - ultima_conexion) > intervalo_conexion)
  { ultima_conexion = millis(); //reinicia a temporización
    sensores_lectura(); //fai unha nova lectura dos sensores
    http_consulta(1);
  } //envía datos a remoto

  // MICROSERVIDOR WEB
  // comproba se hai algún cliente conectado e procesa petición
  http_entrada();

}

//////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// lectura de sensores
// función xeral de lectura de sensores, garda os resultados
// en variables globais do programa
void sensores_lectura()
{
  lee_temperatura_humidade_sht3x(datos); //devolve temperatura=datos[0] e humidade=datos[1]
  //  PRES=lee_presion_atmosferica();
  PRES = 0xffff; //valor nulo
  LUM = lee_sensor_luz();
  CO2 = lee_co2(0); //valor de co2 sen filtrar (0) ou filtrado (1)
  //valores nulos de resto de variables
  VAR6 = 0xffff; VAR7 = 0xffff; VAR8 = 0xffff; VAR9 = 0xffff; VAR10 = 0xffff;
}
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////


