//////////////////////////////////////////////////////////////////////////////////////////////////
// FUNCIÓNS DE LECTURA DE SENSORES 
// -----------------------------------------------------------------------------------------------
// Este módulo fai a lectura de sensores
//
#include <stdio.h>
#include <math.h>
#include "MHZ19.h"

#define PIN_FOTOSENSOR A0 //define o pin de lectura do sensor luminoso
//constante de calibración do fotosensor para intensidade en W/m2
//para o fotodiodo BPW34 con 10 W/m2 (aprox 1000lux) 
//o fotodiodo ten unha superficie 7.02mm2 e ynha responsividade 0,25A/W para 550nm (verde)
//sae unha I=20uA, con R=5K resulta aprox. 100mV, cunha constante x10 queda valor=1000.
//NOTA: constante fixa, calibración con potenciómetro (10k)
#define CONSTANTE_FOTOSENSOR 10.0

//////////////////////////////////////////////////////////////////////////////////////////////////
// FUNCIÓNS DE INTERFAZ
// 1) función de lectura de temperatura: unsigned int lee_temperatura();
//    - devolve o valor de temperatura en (xºC+100)*100 
//    - valores: 0ºC->10000, 100ºC->20000, -100ºC->0, se falla devolve 0xffff
//    - non permite valores fora do rango -100..+100
// 2) función de lectura de humidade relativa: unsigned int lee_humidade_relativa();
//    - devolve o valor de humidade en x%*100
//    - valores: 0%->0, 100%->10000, se falla devolve 0xffff
//    - non permite valores fora do rango 0..100
// 3) función de lectura de presión atmosférica: unsigned int lee_presion_atmosferica();
//    - devolve o valor de presión en x*10, se falla devolve 0xffff
//    - non permite valores fora do rango 0..2000
//
//////////////////////////////////////////////////////////////////////////////////////////////////
// CONTROL DE BUS I2C SOFTWARE CON ATMEGA
// Conexións: SDA = A4, SCL = A5
// (i2c softsare, pode usarse calquera pin)
#define I2C_SDA A4 
#define I2C_SCL A5
#define RETARDO_BIT 100 //retardo en microsegundos
#define RETARDO_LEC 10 //retardo en milisegundos
//////////////////////////////////////////////////////////////////////////////////////////////////
/* 
Os portos dos micros atmega non son tipo colector ou drenador aberto con resistencia,
iso dificulta o manexo do bus i2c que está máis pensado para ese tipo de conexión. 
Tamén dificulta a adaptación a diferentes niveis de tensión. 
Os pins dixitais dos micros atmega cando funcionan como entrada poden activar pullups
incorporados (modo INPUT_PULLUP). Por compatibilidade mellor que usen o modo INPUT
de alta impedancia e poñer unha resistencia externa. 
Cando funcionan como saídas os estados alto e baixo son activos (con trasistores), 
o que pode causar unha sobrecarga no bus. Por iso hai que cambiar a modo entrada 
cando un dispositivo remoto vaia tomar control do bus ou cando se vaian recibir datos. 
O funcionamento do bus i2c precisa resistencias de pullup alomenos na liña sda,
inda que mellor que o teñan as dúas para asegurar o estado inactivo cando non hai 
nada conectado. 
En calquera caso se a liña sda está definida como saída é preferible que teña valor 0
para evitar problemas, xa que iso non causa sobrecarga nos remotos (o valor 1 sí!!).
Nas funcións de control i2c é mellor definir o modo das liñas para evitar problemas. 

MELLORA: 
- evitar poñer as liñas scl e sda a 1 como saídas para que non haxa correntes elevadas
se algún dispositivo mantén as liñas a 0. 
- Hai que poñer resistencias de pullup externas para evitar estados indefinidos (igual 
que no i2c normal).
- cando unha das liñas sda ou scl teña que estar a 1 só hai que definila como entrada 
de alta impedancia, a resistencia externa xa define o nivel e evita exceso de corrente.
- cando as liñas estean a 0 se definen como saída e valor 0, se non se fai así queda 
en estado de alta impedancia sen valor definido.
- RESUMEN: poñer o estado das DÚAS liñas (scl e sda) a 0, e oonmutar o tipo de saida: 
INPUT para valor 1, OUTPUT para valor 0. 

2019: 
Uso da resistencia interna de pullup:
- cando se define unha liña como saída se desactiva o pullup
- cando se define unha liña como entrada e se escribe 1 nela sempre activa o pullup
- o bit PUD (desactiva todos os pullup) non se usa en Arduino, o control é individual
- a instrución pinMode(x, INPUT_PULLUP) fai as dúas cousas, escribe 1 e define como entrada. Se
despois escribimos 0 nesa liña desactiva o pullup, non facelo.  
- Para clk: cando está en 1 se define como INPUT_PULLUP
- Para pasar de 1 a 0 escribir 0 e cambiar a OUTPUT (queda flotante durante un tempo). 
- Para pasar de 0 a 1 (pulso activo) simplemente cambiar a INPUT_PULLUP. 
 
// */
//
// Para o sensor barométrico BMPXXX uso partes de código de sparkfun:
// https://github.com/sparkfun/BMP180_Breakout_Arduino_Library
// http://wmrx00.sourceforge.net/Arduino/BMP085-Calcs.pdf
//
//////////////////////////////////////////////////////////////////////////////////////////////////
// variable que vale 0 se non hai erro e 1 se hai
unsigned char i2c_result=0; //inicializa a variable

// devolve resultado (0=ok)
int i2c_lee_status()
{
  return i2c_result;
}
// reinicia estado
void i2c_reset_status()
{
  i2c_result=0;
}
//////////////////////////////////////////////////////////////////////////////////////////////////
// liña a 1: definir como INPUT e valor 1 (activa pullup interno)
// liña a 0: definir como OUTPUT e valor 0
void i2c_sda0()
{
  digitalWrite(I2C_SDA,0);  //escribir 0 desactiva pullup interno!!
  pinMode(I2C_SDA, OUTPUT); //transistor a masa forza valor 0
  delayMicroseconds(RETARDO_BIT);
}
//////////////////////////////////////////////////////////////////////////////////////////////////
// liña a 1: definir como INPUT e valor 1 (activa pullup interno)
// liña a 0: definir como OUTPUT e valor 0
void i2c_sda1()
{
  pinMode(I2C_SDA, INPUT_PULLUP); //pullup forza valor 1
  delayMicroseconds(RETARDO_BIT);
}
//////////////////////////////////////////////////////////////////////////////////////////////////
// liña a 1: definir como INPUT e valor 1 (activa pullup interno)
// liña a 0: definir como OUTPUT e valor 0
void i2c_scl0()
{
  digitalWrite(I2C_SCL,0);  //escribir 0 desactiva pullup interno!!
  pinMode(I2C_SCL, OUTPUT); //transistor a masa forza valor 0 
  delayMicroseconds(RETARDO_BIT);
}
//////////////////////////////////////////////////////////////////////////////////////////////////
// liña a 1: definir como INPUT e valor 1 (activa pullup interno)
// liña a 0: definir como OUTPUT e valor 0
void i2c_scl1()
{
  pinMode(I2C_SCL, INPUT_PULLUP); //pullup forza valor 1 
  delayMicroseconds(RETARDO_BIT);
}
//////////////////////////////////////////////////////////////////////////////////////////////////
// comenza con sda=scl=1 (input con pullup interno)
// start: con scl=1, sda=1 -> 0
// finaliza con sda=scl=0
void i2c_start()
{
  //scl=sda=1 (xa deberían estar así)
  i2c_scl1(); i2c_sda1();
  //sda->0
  i2c_sda0();
  //scl->0
  i2c_scl0();
}

//////////////////////////////////////////////////////////////////////////////////////////////////
// comenza con sda=0, scl=0
// stop: scl=1, sda=0->1
// finaliza con sda=scl=1 (input con pullup interno)
void i2c_stop()
{
  //sda=scl=0 (por se non estaban)
  i2c_scl0(); i2c_sda0();
  i2c_scl1(); //scl->1
  i2c_sda1(); //sda->1
  // sda=scl=1 (bus i2c inactivo con pullups externos, 
  // iso evita excesos de corrente se outro dispositivo control
  // unha das liñas e a pon a 0)
}

//////////////////////////////////////////////////////////////////////////////////////////////////
// ck->1->0 (supón ck=0 inicialmente e acaba tamén a 0)
void i2c_scl_pulso()
{
  i2c_scl0(); //scl=0 (xa tiña que estar)
  i2c_scl1(); // scl->1
  i2c_scl0(); // scl->0
  // scl queda a 0 para o seguinte pulso ou stop
}
//////////////////////////////////////////////////////////////////////////////////////////////////
// devolve o valor do bit sda sen cambiar sck
bool i2c_consulta_sda()
{
  i2c_sda1(); //sda=1 (libera sda para lectura
  return digitalRead(I2C_SDA);
}
//////////////////////////////////////////////////////////////////////////////////////////////////
// pon sda=1 para permitir que o active o remoto
// scl debería estar a 0 na entrada
// a lectura do dato ten que facerse con scl=1
// scl->1 (lee sda) scl->0 (pulso de scl)
bool i2c_leebit()
{
  bool res;

  i2c_sda1(); //sda=1 (libera sda para lectura
  i2c_scl1(); // scl->1
  res=digitalRead(I2C_SDA); //lectura do valor de sda
  i2c_scl0(); // scl->0

  return res;
}
//////////////////////////////////////////////////////////////////////////////////////////////////
void i2c_orde(unsigned char dato)
{
  if(dato&0x80) i2c_sda1(); else i2c_sda0(); i2c_scl_pulso();
  if(dato&0x40) i2c_sda1(); else i2c_sda0(); i2c_scl_pulso();
  if(dato&0x20) i2c_sda1(); else i2c_sda0(); i2c_scl_pulso();
  if(dato&0x10) i2c_sda1(); else i2c_sda0(); i2c_scl_pulso();
  if(dato&0x08) i2c_sda1(); else i2c_sda0(); i2c_scl_pulso();
  if(dato&0x04) i2c_sda1(); else i2c_sda0(); i2c_scl_pulso();
  if(dato&0x02) i2c_sda1(); else i2c_sda0(); i2c_scl_pulso();
  if(dato&0x01) i2c_sda1(); else i2c_sda0(); i2c_scl_pulso();
}

//////////////////////////////////////////////////////////////////////////////////////////////////
unsigned char i2c_dato()
{
  bool b;
  unsigned char res;
  
  res=0;  
  b=i2c_leebit(); res=res|b; res=res<<1;
  b=i2c_leebit(); res=res|b; res=res<<1;
  b=i2c_leebit(); res=res|b; res=res<<1;
  b=i2c_leebit(); res=res|b; res=res<<1;
  b=i2c_leebit(); res=res|b; res=res<<1;
  b=i2c_leebit(); res=res|b; res=res<<1;
  b=i2c_leebit(); res=res|b; res=res<<1;
  b=i2c_leebit(); res=res|b; 

  return res;
}

//////////////////////////////////////////////////////////////////////////////////////////////////
// SENSOR TEMPERATURA E HUMIDADE SHT1X
//////////////////////////////////////////////////////////////////////////////////////////////////
/* nota: ver datasheet SHT1X 

- sht start (secuencia de inicio do comunicación): 
  con sda&ck=1,  sda 1->0, sck 1->0->1 e sda->1 (quedan a 1)
- antes de enviar bits sck->0 para poder mandar o pulso de subida
- reset conexión: cando se perde a comunicación
  sda=1, enviar 9 ou máis pulsos de sck, deixar sck=1 e enviar 
- soft reset: orden 00011110, o sensor devolve ack no pulso 9 de ck
e hai que esperar polo menos 11ms.

Listaxe de ordes:
Cada orden ten 3 bits de dirección (000) e 5 de orden
- Medida temp/hum: código 00011 temp, 00101 hum
ten que esperar a que remate a medida (11ms-210ms)
isto se indica por un nivel baixo en sda
- mellor facer unha espera temporizada para evitar bloqueo
se falla o sensor ou non está conectado
- O número de bits da conversión se controla no rexistro status
e pode ser 8/12 bits para humidade e 12/14 bits para temperatura

//*/
// listaxe de ordes, os 3 bits superiores son a dirección (000)
#define SHT1X_SOFT_RESET  0x1E
#define SHT1X_MEDIDA_TEMP 0x03
#define SHT1X_MEDIDA_HUM  0x05
#define SHT1X_STATUS_RD   0x07
#define SHT1X_STATUS_WR   0x06
#define SHT1X_DELAY_RESET 20 //tempo de espera para reset (mínimo 11ms)

//coeficientes de corrección de temperatura e humidade
//(quitar os comentarios dos valores correctos)

//coeficientes de humidade
#define SHT1X_HUM_C1 -4.0    //12bits
#define SHT1X_HUM_C2 0.0405  //12bits
#define SHT1X_HUM_C3 -2.8E-6 //12bits
#define SHT1X_HUM_T1 0.01    //12bits
#define SHT1X_HUM_T2 0.00008 //12bits

//#define SHT1X_HUM_C1  -4.0   //8bits
//#define SHT1X_HUM_C2  0.648  //8bits
//#define SHT1X_HUM_C3 -7.2E-4 //8bits
//#define SHT1X_HUM_T1 0.01    //8bits
//#define SHT1X_HUM_T2 0.00128 //8bits

//coeficientes de temperatura segundo a tensión de alimentación
#define SHT1X_TEMP_D1 -40.00  //+5V
//#define SHT1X_TEMP_D1 -39.75  //+4V
//#define SHT1X_TEMP_D1 -39.66  //+3.5V
//#define SHT1X_TEMP_D1 -39.60  //+3V
//#define SHT1X_TEMP_D1 -39.55  //+2.5V

#define SHT1X_TEMP_D2 0.01 //14bit
//#define SHT1X_TEMP_D2 0.04 //12bit

//////////////////////////////////////////////////////////////////////////////////////////////////
// sda, sck inicalmente a 1, quedan sda=1,sck=0 
// para poder enviar unha orden a continuación
// con sda&ck=1,  sda 1->0, sck 1->0->1 e sda->1
void sht1x_start()
{
  i2c_scl1();  //scl=1
  i2c_sda1();  //sda=1
  i2c_sda0();  //sda->0
  i2c_scl0(); i2c_scl1(); //ck->0->1
  i2c_sda1();  //sda=1
  i2c_scl0();  //scl=0
}
//////////////////////////////////////////////////////////////////////////////////////////////////
// secuencia de reinicio do sensor sht
// con sda&ck=1,  sda 1->0, ck 1->0->1 e sda->1 
// (as dúas liñas quedan a 1)
void sht1x_reset_hard()
{
  i2c_sda1();  //sda=1
  i2c_scl0();  //scl=0
  //10 pulsos ck->1->0
  i2c_scl_pulso(); i2c_scl_pulso(); i2c_scl_pulso(); i2c_scl_pulso(); i2c_scl_pulso(); 
  i2c_scl_pulso(); i2c_scl_pulso(); i2c_scl_pulso(); i2c_scl_pulso(); i2c_scl_pulso(); 
  i2c_scl1();  //scl=1
  //sda=1,scl=1
  //mellor non enviar a secuencia start para que quede o bus libre
  //(xa o fará a seguinte orde)  
}
//////////////////////////////////////////////////////////////////////////////////////////////////
// secuencia de reinicio do sensor sht
// con sda&ck=1,  sda 1->0, ck 1->0->1 e sda->1 
// (as dúas liñas quedan a 1)
void sht1x_reset_soft()
{
  bool res;
  sht1x_start(); //secuencia de inicio (sck queda a 0)
  i2c_orde(SHT1X_SOFT_RESET); //orden soft reset (sck queda a 0)
  //neste punto o sensor pon sda=0 para indicar ack
  res=i2c_leebit(); // ck->1-lee sda-ck->0 (pulso de subida sck)
  if (res) i2c_result=1; //erro
  //neste punto o sensor libera sda (=1)
  delay(SHT1X_DELAY_RESET); //retardo para reinicio
  //libera liñas do bus i2c
  i2c_scl1();  //scl=1, sda xa estaba
}
//////////////////////////////////////////////////////////////////////////////////////////////////
// orden de conversion e lectura de datos
// parámetro: temperatura=sht1x_MEDIDA_TEMP, humidade=sht1x_MEDIDA_HUM
// se falla devolve 0xffff
unsigned int sht1x_medida(unsigned char orden)
{
  bool res;
  unsigned char cont,datol,datoh;
  unsigned int valor;

  sht1x_start(); //secuencia de inicio (sck queda a 0)
  i2c_orde(orden); //orden (sck queda a 0)
  //neste punto o sensor pon sda=0 para indicar ack
  res=i2c_leebit(); // ck->1-lee sda-ck->0 (pulso de subida sck)
  // no flanco de baixada de sck o sensor pon sda a 1 para indicar
  // o comenzo do proceso de medida. Cando remata volve poñer sda a 0
  // hai qeu consultar este bit para comprobar que remata a conversión
  // unha vez rematada xa pode comenzarse a lectura de resultados
  if (res) i2c_result=1; //erro
  if(res==0) //se ack (orden aceptada) fai a temporización
  {
    for (cont=0;cont<250;cont++)
    { //bucle de consulta, 250 pasos de 1ms = 250ms máximo
      delay(1); // espera en pasos de 1ms
      res=i2c_consulta_sda();
      if (res==0) break;       
    }
  }
  
  // se res=1 indica orden non aceptada ou fin de temporización,
  // en calquera caso o sensor non funciona ou non está conectado
  if (res) 
  {
    i2c_result=1; //erro
    valor=0xffff; //valor devolto nulo e sae
  }
  else //conversión ok, continúa coa lectura de datos
  {
    //lectura 8 bits do MSB
    i2c_sda1(); //sda=1 para recibir os datos do sensor (xa estaba)
    datoh=i2c_dato();
    i2c_sda0(); //sda=0 significa ack (acepta dato)
    i2c_scl_pulso(); //envía pulso de clk
    
    //lectura 8 bits do LSB
    i2c_sda1(); //sda=1 para recibir os datos do sensor (xa estaba)
    datol=i2c_dato();
    i2c_sda1(); //sda=1 significa nack, interrumpe comunicación
    i2c_scl_pulso(); //envía pulso de clk
    i2c_scl1();  //libera bus, scl=1, sda xa estaba
    //calcula valor de temperatura e devolve resultado
    valor=(unsigned int)datoh;
    valor=valor<<8; //despraza valor
    valor+=(unsigned int)datol;
  }

  //libera liñas do bus i2c
  i2c_scl1(); i2c_sda1(); //libera bus (scl=sda=1)

  //verifica que sda=1, se non a liña sda pode estar a masa sempre
  res=i2c_consulta_sda();
  if (res==0)
  { i2c_result=1; //erro
    valor=0xffff; } //valor devolto nulo e sae
  
  return valor; //devolve o valor calculado ou erro (0xffff)

}

//////////////////////////////////////////////////////////////////////////////////////////////////
// lectura de temperatura 
// devolve o valor de temperatura en (xºC+100)*100 
// valores: 0ºC->10000, 100ºC->20000, -100ºC->0
// non permite valores fora do rango -100..+100
// se falla devolve 0xffff
unsigned int lee_temperatura_sht1x()
{
  unsigned int valor;
  double temp;

  //LECTURA SENSOR
  i2c_reset_status(); //borra indicador de erro i2c
  sht1x_reset_hard();
  valor=sht1x_medida(SHT1X_MEDIDA_TEMP);
  temp=(double)valor;
  temp = SHT1X_TEMP_D1 + temp*SHT1X_TEMP_D2;
  temp = (temp+100)*100;
  //se o valor está fora do rango -100..100 devolve erro
  if((temp<0)||(temp>20000)) temp=0xffff;

  //se había erro devolve 0xffff
  if(valor==0xffff) temp=0xffff;
 
  valor=(unsigned int)temp;

  return valor; 
}
//////////////////////////////////////////////////////////////////////////////////////////////////
// lectura de humidade relativa 
// devolve o valor de humidade en x%*100 
// valores: 0%->0, 100%->10000
// non permite valores fora do rango 0..100
// se falla devolve 0xffff
unsigned int lee_humidade_relativa_sht1x()
{
  unsigned int valor;
  double temp, hum;

  //LECTURA SENSOR
  //borra indicador de erro i2c
  i2c_reset_status();
  sht1x_reset_hard();
  //primeiro lee temperatura para corrección de hr
  valor=sht1x_medida(SHT1X_MEDIDA_TEMP);
  temp=(double)valor;
  temp = SHT1X_TEMP_D1 + temp*SHT1X_TEMP_D2;
  //a continuación lee humidade
  valor=sht1x_medida(SHT1X_MEDIDA_HUM);
  hum=(double)valor;
//  hum=SHT1X_HUM_C1 + hum*SHT1X_HUM_C2 + hum*hum*SHT1X_HUM_C3;
  hum=(temp-25.0)*(SHT1X_HUM_T1 + SHT1X_HUM_T2*hum) + SHT1X_HUM_C1 + hum*SHT1X_HUM_C2 + hum*hum*SHT1X_HUM_C3;
  hum = hum*100;
  //se o valor está fora do rango 0..100 devolve erro
  if((hum<0)||(hum>10000)) hum=0xffff;

  //se había erro devolve 0xffff
  if(valor==0xffff) hum=0xffff;
 
  valor=(unsigned int)hum;

  return valor; 
}
//////////////////////////////////////////////////////////////////////////////////////////////////
// SENSOR PRESIÓN ATMOSFÉRICA E TEMPERATURA BMPXXX
//////////////////////////////////////////////////////////////////////////////////////////////////
// variables do sensor BMP085-180
int bmp_AC1,bmp_AC2,bmp_AC3,bmp_B1,bmp_B2,bmp_MB,bmp_MC,bmp_MD;
unsigned int bmp_AC4,bmp_AC5,bmp_AC6;
double bmp_c3,bmp_c4,bmp_c5,bmp_c6,bmp_mc,bmp_md,bmp_x0,bmp_x1,bmp_x2;
double bmp_y0,bmp_y1,bmp_y2,bmp_p0,bmp_p1,bmp_p2,bmp_b1;
//variable que indica lectura correcta de calibración
bool bmp_cal_ok;

// definicións de ordes do sensor BMPXXX
#define BMP_REX_CONTROL 0xf4
#define BMP_REX_RESULT  0xf6
//dirección base de rexistros de calibración
//(son 22 bytes consecutivos)
#define BMP_REX_CALIBRA 0xaa 
//códigos de escritura e lectura
#define BMP_ORDE_ESC    0xee
#define BMP_ORDE_LEC    0xef
//tempo de conversión de temperatura: 4.5ms
#define BMP_LEE_TEMP    0x2e
//tempo e número de mostras (oversampling)
//0x34=4.5ms, 1 mostra; 0x74=7.5ms, 2 mostras;
//0xB4=13.5ms, 4 mostras;  0xF4=25.5ms, 8 mostras
#define BMP_LEE_PRES0   0x34
#define BMP_LEE_PRES1   0x74
#define BMP_LEE_PRES2   0xB4
#define BMP_LEE_PRES3   0xF4
#define BMP_LEE_PRES BMP_LEE_PRES3

//////////////////////////////////////////////////////////////////////////////////////////////////
// orde inicio temperatura
// par: 0x2e=temp, 0xn4=pres 
//ack=0 ok, ack=1 erro
void bmp_ini_medida(unsigned char par)
{
  bool ack,res;

  i2c_start(); 
  i2c_orde(BMP_ORDE_ESC); //orde escritura
  ack=i2c_leebit(); if (ack) i2c_result=1; //erro
  i2c_orde(BMP_REX_CONTROL); //número de rexistro
  ack=i2c_leebit(); if (ack) i2c_result=1; //erro
  i2c_orde(par);
  ack=i2c_leebit(); if (ack) i2c_result=1; //erro
  i2c_stop(); 

  //verifica que sda=1, se non a liña sda pode estar a masa sempre
  res=i2c_consulta_sda();
  if (res==0) i2c_result=1; //erro

}
//////////////////////////////////////////////////////////////////////////////////////////////////
// lectura rexistro 16 bit (msb-lsb)
// ack=0 ok, ack=1 erro
// nota: a presión atmosférica pode ter un terceiro byte para mellorar
// a precisión, pero non é imprescindible utilizalo
unsigned int bmp_lee_rex16(unsigned char dir)
{
  bool ack,res;
  unsigned char dato;
  unsigned int ival;

  i2c_start();  
  i2c_orde(BMP_ORDE_ESC); //orde escritura
  ack=i2c_leebit(); if (ack) i2c_result=1; //erro
  i2c_orde(dir); //dir rexistros
  ack=i2c_leebit(); if (ack) i2c_result=1; //erro
  i2c_stop(); 

  i2c_start();  
  i2c_orde(BMP_ORDE_LEC); //orde lectura
  ack=i2c_leebit(); if (ack) i2c_result=1; //erro
  dato=i2c_dato(); //byte superior
  ival=(unsigned int)dato;
  ival=ival<<8; //despraza resultado
  i2c_sda0(); i2c_scl_pulso(); //acepta dato
  dato=i2c_dato(); //byte inferior
  ival+=(unsigned int)dato;
  i2c_sda1(); i2c_scl_pulso(); //non acepta dato
  i2c_stop(); 

  //verifica que sda=1, se non a liña sda pode estar a masa sempre
  res=i2c_consulta_sda();
  if (res==0) i2c_result=1; //erro

  return ival;
}

//////////////////////////////////////////////////////////////////////////////////////////////////
// lectura rexistros de calibración do sensor BMPXXX
void bmp_lee_cal()
{
  unsigned int valor;

  //lectura eeprom (rexistros 0xaa-0xbf)
  //(as variables AC4-6 son uint)
  valor=bmp_lee_rex16(BMP_REX_CALIBRA+0x00); bmp_AC1=(int)valor;
  valor=bmp_lee_rex16(BMP_REX_CALIBRA+0x02); bmp_AC2=(int)valor; 
  valor=bmp_lee_rex16(BMP_REX_CALIBRA+0x04); bmp_AC3=(int)valor;
  valor=bmp_lee_rex16(BMP_REX_CALIBRA+0x06); bmp_AC4=valor;
  valor=bmp_lee_rex16(BMP_REX_CALIBRA+0x08); bmp_AC5=valor;
  valor=bmp_lee_rex16(BMP_REX_CALIBRA+0x0a); bmp_AC6=valor;
  valor=bmp_lee_rex16(BMP_REX_CALIBRA+0x0c); bmp_B1=(int)valor;
  valor=bmp_lee_rex16(BMP_REX_CALIBRA+0x0e); bmp_B2=(int)valor;
  valor=bmp_lee_rex16(BMP_REX_CALIBRA+0x10); bmp_MB=(int)valor;
  valor=bmp_lee_rex16(BMP_REX_CALIBRA+0x12); bmp_MC=(int)valor;
  valor=bmp_lee_rex16(BMP_REX_CALIBRA+0x14); bmp_MD=(int)valor;

  // calcula coeficientes
  // código de sparkfun:
  // https://github.com/sparkfun/BMP180_Breakout_Arduino_Library
  // http://wmrx00.sourceforge.net/Arduino/BMP085-Calcs.pdf
 
  bmp_c3 = 160.0 * pow(2,-15) * bmp_AC3;
  bmp_c4 = pow(10,-3) * pow(2,-15) * bmp_AC4;
  bmp_b1 = pow(160,2) * pow(2,-30) * bmp_B1;
  bmp_c5 = (pow(2,-15) / 160) * bmp_AC5;
  bmp_c6 = bmp_AC6;
  bmp_mc = (pow(2,11) / pow(160,2)) * bmp_MC;
  bmp_md = bmp_MD / 160.0;
  bmp_x0 = bmp_AC1;
  bmp_x1 = 160.0 * pow(2,-13) * bmp_AC2;
  bmp_x2 = pow(160,2) * pow(2,-25) * bmp_B2;
  bmp_y0 = bmp_c4 * pow(2,15);
  bmp_y1 = bmp_c4 * bmp_c3;
  bmp_y2 = bmp_c4 * bmp_b1;
  bmp_p0 = (3791.0 - 8.0) / 1600.0;
  bmp_p1 = 1.0 - 7357.0 * pow(2,-20);
  bmp_p2 = 3038.0 * 100.0 * pow(2,-36);
  
}

//////////////////////////////////////////////////////////////////////////////////////////////////
// sensor bmp: lectura de presión atmosférica 
// devolve o valor de presión en x*10 
// non permite valores fora do rango 0..2000
// se falla devolve 0xffff
unsigned int lee_presion_atmosferica()
{
  unsigned int valor;
  double tu, a, T, P;
  double pu,s,x,y,z;
  double pres;
  int res;

  //LECTURA SENSOR
  //borra indicador de erro i2c
  i2c_reset_status();

  //de momento lee sempre coeficientes de calibración
  bmp_lee_cal();

  //lectura valor temperatura
  bmp_ini_medida(BMP_LEE_TEMP); 
  delay(RETARDO_LEC);
  valor=bmp_lee_rex16(BMP_REX_RESULT); //lectura resultado

  tu = (double)valor;
  a = bmp_c5 * (tu - bmp_c6);
  T = a + (bmp_mc / (a + bmp_md));

  //lectura valor presión
  bmp_ini_medida(BMP_LEE_PRES);
  delay(RETARDO_LEC);
  valor=bmp_lee_rex16(BMP_REX_RESULT); //lectura resultado

  pu = (double)valor;
  s = T - 25.0;
  x = (bmp_x2 * pow(s,2)) + (bmp_x1 * s) + bmp_x0;
  y = (bmp_y2 * pow(s,2)) + (bmp_y1 * s) + bmp_y0;
  z = (pu - x) / y;
  P = (bmp_p2 * pow(z,2)) + (bmp_p1 * z) + bmp_p0;
  pres = P*10;
  
  //se o valor está fora do rango 0..2000 devolve erro
  if((pres<0)||(pres>20000)) pres=0xffff;

  //se había erro devolve 0xffff
  res=i2c_lee_status();
  if(res) pres=0xffff;
 
  valor=(unsigned int)pres;

  return valor; 
}
//////////////////////////////////////////////////////////////////////////////////////////////////
// lectura de sensor luminoso
// condicións iniciais: referencia convertidor AD 1.1V
void inicio_sensor_luz()
{
  //establece referencia de tensión en 1.1V
  analogReference(INTERNAL);
}
//////////////////////////////////////////////////////////////////////////////////////////////////
// lectura de sensor luminoso
// devolve a intensidade luminosa en W/m2 
// valores no rango 0..1000, se falla devolve 0xffff
unsigned int lee_sensor_luz()
{
  unsigned int valor;
  float fval;
  valor=analogRead(PIN_FOTOSENSOR);
  fval=(float)valor;
  fval=fval*CONSTANTE_FOTOSENSOR;
  valor=(unsigned int)fval;
  return valor;
}
//////////////////////////////////////////////////////////////////////////////////////////////////
// SENSOR CO2 MH-Z19
//////////////////////////////////////////////////////////////////////////////////////////////////
//
// porto serie virtual:
// velocidades admitidas normalizadas 300-115200 (non valores intermedios)
// funciona en modo 8-N-1 únicamente pero ten capacidade de invertir os 
// niveis lóxicos
#define SERIE_VIRTUAL_PIN_TX A1
#define SERIE_VIRTUAL_PIN_RX A2
#define SERIE_VIRTUAL_VELOCIDADE 9600
//#define SERIE_VIRTUAL_INV true //invirte niveis
#define SERIE_VIRTUAL_INV false //non invirte (modo normal)
#define PIN_CALIBRACION_CO2 A3 //pin usado para calibración do sensor

// crea porto virtual cos parámetros especificados
SoftwareSerial serie_virtual(SERIE_VIRTUAL_PIN_RX, SERIE_VIRTUAL_PIN_TX,SERIE_VIRTUAL_INV);
MHZ19 sensor_co2_mhz19; //crea obxecto sensor mh-z19

//////////////////////////////////////////////////////////////////////////////////////////////////
// función que inicia o sensor de co2 MH-Z19
// - crea o porto serie virtual cos parámetros indicados
// - asocia o porto serie co sensor
// - intenta comunicar e se non o consegue indica erro
void inicio_co2() 
{

// a saida de datos do módulo radio non pode conectarse a unha entrada con pull-up baixo 
// porque sube demasiado os niveis de tensión
//  pinMode(SERIE_VIRTUAL_PIN_RX,INPUT_PULLUP);
  pinMode(SERIE_VIRTUAL_PIN_RX,INPUT);
  pinMode(SERIE_VIRTUAL_PIN_TX,OUTPUT);

  //inicializa porto serie virtual (non devolve nada)
  serie_virtual.begin(SERIE_VIRTUAL_VELOCIDADE);

  //asocia o sensor co porto serie virtual e inicia comunicacións
  sensor_co2_mhz19.begin(serie_virtual);

  //se o sensor comunica imprime parámetros
  if(sensor_co2_chk()==RESULT_OK)
  {
    // Autocalibración OFF=false, ON=true
    sensor_co2_mhz19.autoCalibration(false);        

    //imprime estado sensor CO2
    Serial.print(F("SENSOR MH-Z19"));
    Serial.print(F("Rango de medida: "));
    Serial.println(sensor_co2_mhz19.getRange());   
    Serial.print(F("Valor de fondo CO2: "));
    Serial.println(sensor_co2_mhz19.getBackgroundCO2());
    Serial.print(F("Temperatura: "));
    Serial.println(sensor_co2_mhz19.getTempAdjustment());
    Serial.print(F("Estado ABC: ")); 
    ( sensor_co2_mhz19.getABC() ? Serial.println("ON") :  Serial.println("OFF") );
  }
  else
    Serial.println(F("Erro sensor MH-Z19"));

}

//////////////////////////////////////////////////////////////////////////////////////////////////
// función que verifica se o sensor está presente
byte sensor_co2_chk()
{
  // verifica comunicación co sensor e actualiza código de erro
  sensor_co2_mhz19.verify(); // usa función verify() para comprobar enlace
//  sensor_co2_mhz19.getABC(); // usa función getabc() para comprobar enlace
  return sensor_co2_mhz19.errorCode;
}
//////////////////////////////////////////////////////////////////////////////////////////////////
// lectura de sensor co2 MH-Z19
// devolve a concentración de co2 en ppm, valores no rango 400..2000,  se falla devolve 0xffff
// parámetro filtro indica se devolve o valor de lectura sen procesar (=0) ou procesado (=1)
// se está procesado limita o rango 
unsigned int lee_co2(byte filtro)
{
  unsigned int valor;
  int dato;

  valor=0xffff; //indica erro inicialmente
  if(sensor_co2_chk()==RESULT_OK)
  {
    // a función getco2(false,true) forza unha nova lectura, senón devolve a última válida
    // o primeiro parámetro indica lectura limitada (false) ou sen filtrar (true) 
    // o segundo forza a lectura (true) ou non (false)
    /* note: getCO2() default is command "CO2 Unlimited". This returns the correct CO2 reading 
    even if below background CO2 levels or above range (useful to validate sensor). You can use
    the usual documented command with getCO2(false) */  
    if(filtro==0) dato=sensor_co2_mhz19.getCO2(true,true);  // Lectura de CO2 en ppm sen filtrada
    else dato=sensor_co2_mhz19.getCO2(false,true);  // Lectura de CO2 en ppm filtrada
    //valor de retorno 0=erro (devolve 0xffff), senón devolve lectura real
    if(dato!=0) valor=(unsigned int)dato;
  }
  
  return valor;
}  
//////////////////////////////////////////////////////////////////////////////////////////////////
// función que activa o pin de calibración do sensor mh-z19 durante o tempo especificado
// NOTA: o proceso ten que facerse mantendo o sensor en aire libre limpo durante 20 minutos 
// en condicións estables. 
// Hai que manter o pin a nivel baixo durante 7 segundos, para iso 
// 
void sensor_co2_calibracion()
{
  digitalWrite(PIN_CALIBRACION_CO2,0);  //escribir 0 desactiva pullup interno!!
  pinMode(PIN_CALIBRACION_CO2, OUTPUT); //transistor a masa forza valor 0
  delay(10000); //espera 10s
  pinMode(PIN_CALIBRACION_CO2,INPUT_PULLUP); //cambia a entrada e activa pullup (nivel alto)
}
//////////////////////////////////////////////////////////////////////////////////////////////////

