//////////////////////////////////////////////////////////////////////////////////////////////////
// SENSOR TEMPERATURA E HUMIDADE SHT3X
//////////////////////////////////////////////////////////////////////////////////////////////////
/* nota: ver datasheet SHT3X 

- sht start (secuencia de inicio do comunicación): 
  con sda&ck=1,  sda 1->0, sck 1->0->1 e sda->1 (quedan a 1)
- antes de enviar bits sck->0 para poder mandar o pulso de subida
- reset conexión: cando se perde a comunicación
  sda=1, enviar 9 ou máis pulsos de sck, deixar sck=1 e enviar 

Listaxe de ordes:
- o sht3x ten ordes de 16 bits. En primeiro lugar vai o byte de enderezo: 7 bits msb (valor
0x44 ou ox45) e un bit de r/w (0=write)
- Medida temp/hum: a lectura pode facerse mediante espera e consulta (polling), enviando a 
orden de lectura se o dispositivo responde nack hai que esperar e cando devolve ack pode
lerse o resultado. Máximo tempo 15ms. Mellor facer unha espera temporizada consultando cada
ms para evitar bloqueo se falla o sensor ou non está conectado
O outro método é o denominado "clock stretching". Neste caso despóis da orden de adquisición
se envía a orden de lectura e o dispositivo fai dúas cousas: responde ack (sda=0) e mantén a 
nivel baixo scl (clock). Cando o maestro mando o pulso sck para consultar o bit ack ten que 
verificar que a liña non se pon a 1, o que indica que o sensor está facendo a adquisición.
Cando remata libera a liña sck e o maestro xa pode facer a lectura. 
Mellor usar o método de consulta que é máis sinxelo e non precisa engadir funcións novas. 

//*/
// listaxe de ordes (códigos de 2 bytes consecutivos, primeiro msb)
// dirección dispositivo con bit r/w
// dirección por defecto 0x44, alternativa 0x45. Ocupan os 7msb do byte, o lsb é r/w (0=write)
#define SHT3X_ADDRESS1_WRITE 0x88
#define SHT3X_ADDRESS1_READ  0x88+1
#define SHT3X_ADDRESS2_WRITE 0x8A
#define SHT3X_ADDRESS2_READ  0x8A+1
//orden soft reset, código 0x30a2
#define SHT3X_SOFT_RESET 0X30A2 
//orden break, código 0x3093. Esta orden interrompe a adquisición continua
#define SHT3X_BREAK 0X3093
//orden lectura alta repetibilidade con lectura por consulta (sen clock stretching)
#define SHT3X_LECTURA1 0X2400

#define SHT3X_DELAY_RESET 20 //tempo de espera para reset (mínimo 11ms)

//coeficientes de corrección de temperatura e humidade
//(quitar os comentarios dos valores correctos)

//coeficientes de humidade
#define SHT3X_HUM_C1 -4.0    //12bits
#define SHT3X_HUM_C2 0.0405  //12bits
#define SHT3X_HUM_C3 -2.8E-6 //12bits
#define SHT3X_HUM_T1 0.01    //12bits
#define SHT3X_HUM_T2 0.00008 //12bits

//#define SHT3X_HUM_C1  -4.0   //8bits
//#define SHT3X_HUM_C2  0.648  //8bits
//#define SHT3X_HUM_C3 -7.2E-4 //8bits
//#define SHT3X_HUM_T1 0.01    //8bits
//#define SHT3X_HUM_T2 0.00128 //8bits

//variables para almacenar datos de lectura
byte sht3x_temp_low, sht3x_temp_high, sht3x_temp_crc;
byte sht3x_hum_low, sht3x_hum_high, sht3x_hum_crc;

//////////////////////////////////////////////////////////////////////////////////////////////////
// secuencia de reinicio do sensor sht (só comunicacións, non afecta valores)
// con sda&ck=1,  sda 1->0, ck 1->0->1 e sda->1 
// (as dúas liñas quedan a 1)
void sht3x_reset_interface()
{
  i2c_sda1();  //sda=1
  i2c_scl0();  //scl=0
  //10 pulsos ck->1->0
  i2c_scl_pulso(); i2c_scl_pulso(); i2c_scl_pulso(); i2c_scl_pulso(); i2c_scl_pulso(); 
  i2c_scl_pulso(); i2c_scl_pulso(); i2c_scl_pulso(); i2c_scl_pulso(); i2c_scl_pulso(); 
  i2c_scl1();  //scl=1
  //sda=1,scl=1
  //mellor non enviar a secuencia start para que quede o bus libre
  //(xa o fará a seguinte orde)  
}
//////////////////////////////////////////////////////////////////////////////////////////////////
// función de envío de orde ó sensor
// recibe un código de 2 bytes e envía a dirección e código da orde(msb, lsb) 
// consulta o bit ack e rexistra o erro na variable result
void sht3x_orde(unsigned int orde)
{
  bool res;
  byte dato;
  
  i2c_start(); //secuencia de inicio (sck queda a 0)

  i2c_orde(SHT3X_ADDRESS1_WRITE); //envía dirección por defecto con bit w
  //neste punto o sensor pon sda=0 para indicar ack
  res=i2c_leebit(); // ck->1-lee sda-ck->0 (pulso de subida sck)
  if (res) i2c_result=1; //erro

  dato=(byte)(orde>>8);
  i2c_orde(dato); //envía byte alto da orde
  //neste punto o sensor pon sda=0 para indicar ack
  res=i2c_leebit(); // ck->1-lee sda-ck->0 (pulso de subida sck)
  if (res) i2c_result=1; //erro

  dato=(byte)(orde&0xff);
  i2c_orde(dato); //envía byte baixo da orde
  //neste punto o sensor pon sda=0 para indicar ack
  res=i2c_leebit(); // ck->1-lee sda-ck->0 (pulso de subida sck)
  if (res) i2c_result=1; //erro

  i2c_stop(); //secuencia de fin (sck=sda=1)
}
//////////////////////////////////////////////////////////////////////////////////////////////////
// secuencia de reinicio do sensor sht
// con sda&ck=1,  sda 1->0, ck 1->0->1 e sda->1 
// (as dúas liñas quedan a 1)
void sht3x_reset_soft()
{
  sht3x_orde(SHT3X_SOFT_RESET); //envía código de orde (3 bytes)
  delay(SHT3X_DELAY_RESET); //retardo para reinicio
}
//////////////////////////////////////////////////////////////////////////////////////////////////
// fin de adquisición continua do sensor sht
void sht3x_break()
{
  sht3x_orde(SHT3X_BREAK); //envía código de orde (3 bytes)
}
//////////////////////////////////////////////////////////////////////////////////////////////////
// orden de conversion e lectura de datos de temperatura e humidade
// se falla devolve erro en variable result (=1)
// os datos recibidos gárdanse nas variables sht3x_temp_low/high/crc, sht3x_hum_low/high/crc
void sht3x_medida()
{
  bool res;
  byte cont;

  sht3x_orde(SHT3X_LECTURA1);
  // neste punto o sensor inicia o proceso de medida
  // lectura por consulta: hai que enviar a dirección con bit r e verificar ack
  // mentres dura a medida o sensor devolve nack, cando remata devolve ack
  // e pode iniciarse a lectura de datos
  if(i2c_lee_status()) return; //se hai erros non segue coa lectura

  //bucle de consulta para lectura (50 pasos de 1ms = 50ms máximo)
  for (cont=0;cont<50;cont++)
  { delay(1); // espera en pasos de 1ms
    i2c_start(); //secuencia de inicio
    i2c_orde(SHT3X_ADDRESS1_READ); //envía dirección por defecto con bit r
    //neste punto o sensor pon sda=0 para indicar ack ou 1 para nack
    res=i2c_leebit(); // ck->1-lee sda-ck->0 (pulso de subida sck)
    if (res==0) break; //sda=0 indica ack, pode seguir a lectura
    //se sda=1 indica nack, remata a trama con stop e continúa bucle
    i2c_stop(); //secuencia de fin (sck=sda=1)
  }
  // se res=1 indica fin de temporización, devolve erro
  // (sensor non funciona ou non conectado)
  if (res) { i2c_result=1; return; }
  
  //res=0 indica fin de medida ok, continúa a lectura de datos
    
  //lectura temperatura (msb, lsb, crc)
  //primeiro pon sda=1 para lectura de datos, logo lee 8 bits e 
  // por último sda=0 e pulso sck para aceptar dato
  i2c_sda1(); sht3x_temp_high=i2c_dato(); i2c_sda0(); i2c_scl_pulso(); //ack
  i2c_sda1(); sht3x_temp_low =i2c_dato(); i2c_sda0(); i2c_scl_pulso(); //ack
  i2c_sda1(); sht3x_temp_crc =i2c_dato(); i2c_sda0(); i2c_scl_pulso(); //ack
  //lectura humidade relativa (msb, lsb, crc)
  i2c_sda1(); sht3x_hum_high=i2c_dato(); i2c_sda0(); i2c_scl_pulso(); //ack
  i2c_sda1(); sht3x_hum_low =i2c_dato(); i2c_sda0(); i2c_scl_pulso(); //ack
  i2c_sda1(); sht3x_hum_crc =i2c_dato(); i2c_sda1(); i2c_scl_pulso(); //nack
  
  i2c_scl1();  //libera bus, scl=1, sda xa estaba
  
  //verifica que sda=1, se non a liña sda pode estar a masa sempre
  res=i2c_consulta_sda();
  if (res==0) i2c_result=1; //erro
  
  return;
}

//////////////////////////////////////////////////////////////////////////////////////////////////
// lectura de temperatura e humidade relativa
// devolve o valor de temperatura en (xºC+100)*100 
// valores: 0ºC->10000, 100ºC->20000, -100ºC->0 (non permite valores fora do rango -100..+100)
// devolve o valor de humidade en x%*100 
// valores: 0%->0, 100%->10000 (non permite valores fora do rango 0..100)
// devolve os valores na matriz indicada, en dous datos consecutivos (temp, hum)
void lee_temperatura_humidade_sht3x(unsigned int *datos)
{
  unsigned int valor;
  long lval;

  //LECTURA SENSOR SHT3X
  i2c_reset_status(); //borra indicador de erro i2c
  sht3x_reset_interface();
  //lectura combinada de temperatura e humidade
  sht3x_medida();

  //CÁLCULO DE TEMPERATURA
  //usamos unha variable long para traballar con números grandes con signo
  //cun programa máis compacto que con float ou double
  lval=(long)sht3x_temp_high;
  lval=(lval<<8)+(long)sht3x_temp_low;
  lval=lval*17500/65535;
  lval=lval-4500+10000;

  //se o valor está fora do rango -100..100 devolve erro
  if((lval<0)||(lval>20000)) lval=0xffff;

  //se había erro devolve 0xffff
  if(i2c_lee_status()) lval=0xffff;
 
  datos[0]=(unsigned int)lval;

  //CÁLCULO DE HUMIDADE RELATIVA
  lval=(long)sht3x_hum_high;
  lval=(lval<<8)+(long)sht3x_hum_low;
  lval=lval*10000/65535;

  //se o valor está fora do rango 0..100 devolve erro
  if((lval<0)||(lval>10000)) lval=0xffff;

  //se había erro devolve 0xffff
  if(i2c_lee_status()) lval=0xffff;
 
  datos[1]=(unsigned int)lval;
}

